import argparse
import configparser
import json
import logging
import i3ipc

from swayblur import paths
from swayblur.blurManager import BlurManager


# parse and validate the arguments
def parseArgs() -> bool:
    BLUR_MIN = 5
    BLUR_MAX = 100
    ANIMATE_MIN = 1
    ANIMATE_MAX = 20

    parser = argparse.ArgumentParser()
    parser.add_argument('-b', '--blur', type=int, default=20,
            help='The blur strength (default: %(default)d, min: {}, max: {})'.format(BLUR_MIN, BLUR_MAX))
    parser.add_argument('-a', '--animate', type=int, default=1,
            help='Animation duration (default: %(default)d, min: {}, max: {})'.format(ANIMATE_MIN, ANIMATE_MAX))
    parser.add_argument('-c', '--config-path', type=str, default=paths.DEFAULT_OGURI_DIR,
            help='Path to the oguri configuration file to use (default: %(default)s)')
    parser.add_argument('-v', '--verbose', action='store_true',
            help='Prints additional information')
    args = parser.parse_args()

    # Validate args
    if args.blur < BLUR_MIN or args.blur > BLUR_MAX:
        parser.error('Unable to run swayblur, blur is set to %d, which is not between %d-%d' % (args.blur, BLUR_MIN, BLUR_MAX))
    if args.animate < ANIMATE_MIN or args.animate > ANIMATE_MAX:
        parser.error('Unable to run swayblur, animate is set to %d, which is not between %d-%d' % (args.animate, ANIMATE_MIN, ANIMATE_MAX))
    if args.animate > args.blur:
        parser.error('Unable to run swayblur, animate value %d is greater than blur value %d' % (args.animate, args.blur))
    if not paths.exists(args.config_path):
        parser.error('Unable to run swayblur, oguri config %s not found' % (args.config_path))

    return args


# parse the oguri config
def parseConfig(configPath: str) -> dict:
    config = configparser.ConfigParser()
    config.read(configPath)

    # init outputs to their defaults
    outputSettings = {}
    for output in i3ipc.Connection().get_outputs():
        outputSettings[output.name] = {
            'image': '',
            'filter': '',
            'anchor': '',
            'scaling-mode': 'fill',
            'is-blurred': False
        }

    # iterate through each output in the config
    for section in config.sections():
        outputName = section.split('output ')[-1]
        outputsToUpdate = []

        if outputName == '*':
            for output in outputSettings:
                if outputSettings[output]['image'] == '':
                    outputsToUpdate.append(output)
        else:
            outputsToUpdate.append(outputName)

        for output in outputsToUpdate:
            for key in config[section]:
                outputSettings[output][key] = config[section][key]

    return outputSettings


def verifySettingsCache(blurStrength: int, animationDuration: int) -> None:
    try:
        with open(paths.CACHE_VALIDATION_FILE, 'r') as f:
            settings = json.load(f)
            if settings['blur'] == blurStrength and settings['animate'] == animationDuration:
                return
    except FileNotFoundError:
        pass

    # new settings, clear & recreate cache
    logging.info('New swayblur settings found, recreating cache...')
    paths.deleteCache()
    paths.createCache()

    with open(paths.CACHE_VALIDATION_FILE, 'w') as f:
        f.write(json.dumps({
            'blur': blurStrength,
            'animate': animationDuration,
        }))
    return


def configureLogger() -> None:
    logging.basicConfig(format='[%(levelname)s\033[0m] %(message)s')
    logging.addLevelName(logging.INFO, '\033[1;34mI')
    logging.addLevelName(logging.ERROR, '\033[1;31mE')


def main() -> None:
    # parse arguments
    args = parseArgs()

    # setup logger
    configureLogger()
    if args.verbose:
        logging.getLogger().setLevel(logging.INFO)

    # parse oguri config
    outputConfigs = parseConfig(args.config_path)

    # clear cache if the blurStrength / animationDuration have been changed since last run
    verifySettingsCache(args.blur, args.animate)

    # blur the wallpaper
    blurManager = BlurManager(outputConfigs, args.blur, args.animate)
    blurManager.start()


if __name__ == "__main__":
    main()



import pathlib
import shutil

DEFAULT_OGURI_DIR = pathlib.Path.home() / '.config/oguri/config'

CACHE_DIR = pathlib.Path.home() / '.cache/swayblur'
CACHE_VALIDATION_FILE = CACHE_DIR / 'settings.json'

# create the cache dir if it doesn't exist
def createCache() -> None:
    CACHE_DIR.mkdir(parents=True, exist_ok=True)

# delete the cache dir
def deleteCache() -> None:
    shutil.rmtree(CACHE_DIR, ignore_errors=True)

# check if a path exists
def exists(path: str) -> bool:
    return pathlib.Path(path).is_file()

# returns the path to a given frame for a given output
def framePath(hash: str, frame: int) -> str:
    return '%s/%s-%d.png' % (CACHE_DIR, hash, frame)

# returns the path to a cached image based on its path + hash
def cachedImagePath(path: str, hash: str) -> str:
    return CACHE_DIR / ('%s%s' % (hash, pathlib.Path(path).suffix))



import subprocess
import logging

class Output:
    def __init__(self, name: str, wallpaper: str, blurFrames: list, settings: dict) -> None:
        self.name = name
        self.wallpaper = wallpaper
        self.blurFrames = blurFrames
        self.settings = settings
        self.isBlurred = False

    def blur(self) -> None:
        if not self.isBlurred:
            for framePath in self.blurFrames:
                self.switchWallpaper(framePath)
            self.isBlurred = True

    def unblur(self) -> None:
        if self.isBlurred:
            for framePath in self.blurFrames[:0:-1]:
                self.switchWallpaper(framePath)
            self.switchWallpaper(self.wallpaper)
            self.isBlurred = False

    def switchWallpaper(self, path: str) -> None:
        if self.wallpaper: # not all outputs have wallpaper
            try:
                subprocess.run([
                    'ogurictl',
                    'output', self.name,
                    '--image', path,
                    '--filter', self.settings['filter'],
                    '--anchor', self.settings['anchor'],
                    '--scaling-mode', self.settings['scaling-mode']
                ])
                logging.info('Changed output %s wallpaper to %s' % (self.name, path))
            except:
                logging.error('Could not set wallpaper, ensure oguri is installed')
                exit()



import multiprocessing
import subprocess
import filecmp
import shutil
import hashlib
import logging
import i3ipc

from swayblur import paths
from swayblur.output import Output


def genBlurredImage(inputPath: str, outputPath: str, blurLevel: int) -> None:
    try:
        subprocess.run(['convert', inputPath, '-blur', '0x%d' % blurLevel, outputPath])
    except FileNotFoundError:
        logging.error('Could not create blurred version of wallpaper, ensure imagemagick is installed')
        exit()

    logging.info('Generated image %s' % outputPath)


def verifyWallpaperCache(wallpaperPath: str, wallpaperHash: str) -> bool:
    cachedWallpaper = paths.cachedImagePath(wallpaperPath, wallpaperHash)

    if paths.exists(cachedWallpaper) and filecmp.cmp(wallpaperPath, cachedWallpaper):
        logging.info('wallpaper %s is cached as %s' % (wallpaperPath, cachedWallpaper))
        return True

    logging.info('wallpaper %s added to the cache as %s' % (wallpaperPath, cachedWallpaper))
    shutil.copy(wallpaperPath, cachedWallpaper)
    return False


class BlurManager:
    def __init__(self, outputConfigs: dict, blurStrength: int, animationDuration: int) -> None:
        self.SWAY = i3ipc.Connection()
        self.outputs = {}

        animationFrames = [
            (i + 1) * (blurStrength // animationDuration) for i in range(animationDuration)
        ]

        # create an output object for each output in the configuration
        for name in outputConfigs:
            outputCfg = outputConfigs[name]
            outputWallpaper = outputCfg['image']

            if not outputWallpaper: # if output has no wallpaper
                self.outputs[name] = Output(name, '', [], {})
                continue

            imageHash = hashlib.md5(outputWallpaper.encode()).hexdigest()
            cachedImage = paths.cachedImagePath(outputWallpaper, imageHash)

            self.outputs[name] = Output(
                name,
                cachedImage,
                [paths.framePath(imageHash, frame) for frame in animationFrames],
                {
                    'filter': outputCfg['filter'] ,
                    'anchor': outputCfg['anchor'],
                    'scaling-mode': outputCfg['scaling-mode'],
                }
            )

            # check if new wallpaper must be generated
            if not verifyWallpaperCache(outputWallpaper, imageHash):
                print('Generating blurred wallpaper frames')
                print('This may take a minute...')
                with multiprocessing.Pool() as pool:
                    pool.starmap(
                        genBlurredImage,
                        [[cachedImage, paths.framePath(imageHash, frame), frame] for frame in animationFrames]
                    )
                print('Blurred wallpaper generated for %s' % name)
            else:
                print('Blurred wallpaper exists for %s' % name)


    def start(self) -> None:
        print("Listening...")
        self.SWAY.on(i3ipc.Event.WINDOW_MOVE, self.handleMove)
        self.SWAY.on(i3ipc.Event.WINDOW_NEW, self.handleOther)
        self.SWAY.on(i3ipc.Event.WINDOW_CLOSE, self.handleOther)
        self.SWAY.on(i3ipc.Event.WORKSPACE_FOCUS, self.handleOther)
        self.SWAY.main()


    def handleMove(self, _sway: i3ipc.Connection, event: i3ipc.Event) -> None:
        container = self.SWAY.get_tree().find_by_id(event.ipc_data['container']['id'])
        containerOutput = container.workspace().ipc_data['output']

        focusedContainer = self.SWAY.get_tree().find_focused()
        focusedOutput = focusedContainer.workspace().ipc_data['output']

        # window moved to a workspace on a different output
        if container != focusedContainer:
            self.outputs[containerOutput].blur()
            if focusedContainer == focusedContainer.workspace():
                self.outputs[focusedOutput].unblur()
        # window moved to a new workspace on same output
        elif container == container.workspace(): # if workspace is empty
            self.outputs[containerOutput].unblur()


    def handleOther(self, _sway: i3ipc.Connection, _event: i3ipc.Event) -> None:
        focusedContainer = self.SWAY.get_tree().find_focused()
        focusedOutputName = focusedContainer.workspace().ipc_data['output']

        if focusedContainer == focusedContainer.workspace(): # if workspace is empty
            self.outputs[focusedOutputName].unblur()
        else:
            self.outputs[focusedOutputName].blur()
